# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/CalibrationCurve',
                'doc_host': 'https://Rhys-McAlister.github.io',
                'git_url': 'https://github.com/Rhys-McAlister/CalibrationCurve',
                'lib_path': 'CalibrationCurve'},
  'syms': { 'CalibrationCurve.core': { 'CalibrationCurve.core.CalibrationModel': ('core.html#calibrationmodel', 'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.__init__': ( 'core.html#calibrationmodel.__init__',
                                                                                            'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.calculate_sse': ( 'core.html#calibrationmodel.calculate_sse',
                                                                                                 'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.calculate_sxhat': ( 'core.html#calibrationmodel.calculate_sxhat',
                                                                                                   'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.calculate_syx': ( 'core.html#calibrationmodel.calculate_syx',
                                                                                                 'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.calculate_uncertainty': ( 'core.html#calibrationmodel.calculate_uncertainty',
                                                                                                         'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.fit_model': ( 'core.html#calibrationmodel.fit_model',
                                                                                             'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.fit_ols': ( 'core.html#calibrationmodel.fit_ols',
                                                                                           'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.get_params': ( 'core.html#calibrationmodel.get_params',
                                                                                              'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.get_r2': ( 'core.html#calibrationmodel.get_r2',
                                                                                          'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.get_t_value': ( 'core.html#calibrationmodel.get_t_value',
                                                                                               'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.inverse_prediction': ( 'core.html#calibrationmodel.inverse_prediction',
                                                                                                      'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.load_data': ( 'core.html#calibrationmodel.load_data',
                                                                                             'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.plot_fit': ( 'core.html#calibrationmodel.plot_fit',
                                                                                            'CalibrationCurve/core.py'),
                                       'CalibrationCurve.core.CalibrationModel.tabulate_results': ( 'core.html#calibrationmodel.tabulate_results',
                                                                                                    'CalibrationCurve/core.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['CalibrationModel']

# %% ../nbs/00_core.ipynb 5
class CalibrationModel:
    def __init__(self, data, response_variable, test_replicates):
        self.raw_data = self.load_data(data)
        self.response_variable = response_variable
        self.fit = None
        self.test_replicates = test_replicates
        self.cal_line_points = self.raw_data.shape[0]
        self.r2 = None

    def load_data(self, data):
        if ".csv" in data:
            raw_data = pd.read_csv(data)
        else:
            raw_data = data
        return raw_data

    def fit_ols(self):
        X = self.raw_data[["concentration"]]
        y = self.raw_data[self.response_variable]
        self.fit = LinearRegression().fit(X, y)
        self.r2 = self.fit.score(X, y)
        return self.fit

    def get_params(self):
        slope = self.fit.coef_[0]
        intercept = self.fit.intercept_
        return slope, intercept

    def get_r2(self):
        return self.r2

    def inverse_prediction(self, unknown: float):
        slope, intercept = self.get_params()
        return (unknown - intercept) / slope

    def calculate_sse(self):
        y_pred = self.fit.predict(self.raw_data[["concentration"]])
        return np.sum((y_pred - self.raw_data[self.response_variable]) ** 2)

    def calculate_syx(self):
        return np.sqrt((self.calculate_sse()) / (len(self.raw_data) - 2))

    def get_t_value(self, alpha):
        return t.ppf(1 - alpha / 2, len(self.raw_data) - 2)

    def calculate_uncertainty(self):
        return self.calculate_sxhat() * self.get_t_value(0.05)

    def calculate_sxhat(self):
        return (self.calculate_syx() / self.fit.coef_[0]) * np.sqrt(1 / self.test_replicates + self.cal_line_points)

    def fit_model(self):
        self.fit_ols()
        self.get_params()
        self.get_r2()
        self.calculate_uncertainty()
        self.tabulate_results()

    def plot_fit(self):
        sns.regplot(x="concentration", y=self.response_variable, data=self.raw_data)
        sns.despine()
        sns.set_context("paper")
        plt.title(f"Calibration curve of concentration versus {self.response_variable}")
        plt.xlabel("Concentration")
        plt.ylabel("Peak Value")
        plt.annotate(f"R-squared = {self.get_r2():.3f}", xy=(0.3, 0.8), xycoords="axes fraction", fontsize=9,
                     ha="center", va="center")
        plt.annotate(f"Regression formula: y = {self.get_params()[0]:.3f} * x + {self.get_params()[1]:.3f}",
                     xy=(0.3, 0.7), xycoords="axes fraction", fontsize=9, ha="center", va="center")
        plt.show()



    def tabulate_results(self):
        print(f"Calibration curve of {self.response_variable} versus concentration")
        print(f"R2 = {self.r2}")
        print(f"Slope = {self.get_params()[0]}")
        print(f"Intercept = {self.get_params()[1]}")
        print(f"Uncertainity = {self.calculate_uncertainty()}")
        # print(f"Prediction = {self.inverse_prediction()}")
    
